;
; MachZ bootstrap jumpers configuration display utility
;
; (c)2000 ZF Linux Devices, Inc.
;
; This code executes as BUR "Load and execute" function. 
;

.MODEL TINY
.CODE
.486p

		org	0
START:			
		push	cs
		pop	es
		
		push	cs
		pop	ds
		
		mov	di, offset HelloText	; ES:DI - text to show
		xor	cx, cx			; display until 0 reached
		call	SerSend
		call	CRLF
		
		mov	dx, 218h
		mov	al, 62h		
		out	dx, al
		mov	dx, 21Ah
		in	eax, dx			; EAX is straps now
		mov	ebp, eax		; save bits
		
		mov	di, offset StrapText
		xor	cx, cx
		call	SerSend
		call	SerOut32
		call	CRLF
		call	CRLF
		
		mov	bx, offset Straps	
StrapsLoop:		
		cmp	byte ptr ds:[bx.sBits], 0FFh
		jz	AllDone

		; Show strap name
		
		mov	di, bx
		add	di, sName
		xor	cx, cx
		call	SerSend
		
		; Place useful bit(s) to CL
		
		mov	cx, bp				; restore bits
		xor	ch, ch
		and	cl, byte ptr ds:[bx.sMask]	; leave only useful bits
		
		; Caclulate option text shift to AX
					
		mov	ax, sOption1-sOption0		; get length of one option
		mul	cl				; seek to option
		
		; Display bit(s) value(s)
		
		push	ax
		mov	al, ' '
		call	SerSend2
		mov	dx, cx				; save bits to DX
		mov	cl, byte ptr ds:[bx.sBits]
		xor	ch, ch
		ror	dx, cl
@@:
		rol	dx, 1
		mov	ax, dx
		and	ax, 1
		add	ax, 30h
		call	SerSend2
		loop	@b
		
		mov	cl, 4				; pad with spaces to align options
		sub	cl, byte ptr ds:[bx.sBits]	
		mov	al, ' '
@@:
		call	SerSend2
		loop	@b
		pop	ax
		
		; prepare EBP for next iteration
		
		mov	cl, byte ptr ds:[bx.sBits]
		shr	ebp, cl				
		
		; Seek to options iniside record
		
		mov	di, bx
		add	di, sOption0			; point to first option text at record
		
		cmp	byte ptr es:[di], 0		; if 0, don't try to show anything
		jz	@f
		
		add	di, ax				; seek to correct option text
						
		xor	cx, cx
		call	SerSend				; show option
@@:		
		add	bx, STRAPLEN			; seek to next record
		call	CRLF
		jmp	StrapsLoop		
AllDone:
		call	CRLF
		mov	di, offset SpeedText
		xor	cx, cx
		call	SerSend				; Show "Current CPU speed: "
				
		call	MeasureSpeed			; get CPU speed into EAX (in kHz*10)
		call	ShowDecimal
	
		mov	di, offset MHzText
		xor	cx, cx
		call	SerSend
		call	CRLF
		
		call	ZTPrepareRead		; always leave with ZFL in Read mode
		retf				; resume with BUR
	
;
; Display value in EAX in decimal
;
; Entry: EAX - value
;
; Exit: none
;
; Uses: all
;

ShowDecimal	proc
		xor	di, di
		mov	ecx, 10000000
		push	eax
NextDigitBegin:				
		xor	edx, edx
		div	ecx
		cmp	di, 0			; is that a first digit printed?
		jnz	@f			; if not a first digit printed, skip leading '0' check
		cmp	al, 0
		jz	KillDigit
@@:
		add	al, 30h			; convert to ascii
		call	SerSend2
		inc	di			; increment digit counter
		sub	al, 30h			; back to number
		
		cmp	ecx, 100		; check if we need to print out decimal point
		jnz	KillDigit
		push	eax
		mov	al, '.'
		call	SerSend2
		pop	eax
KillDigit:
		mul	ecx
		pop	edx			; was EAX when first pushed
		sub	edx, eax		; substract one decimal place
		cmp	ecx, 1			; last decimal place?
		jz	AllPrinted
		push	edx			; current number left
		push	edx
		mov	eax, ecx
		xor	edx, edx
		mov	ecx, 10
		div	ecx
		mov	ecx, eax		; divisor back in place
		pop	eax
		jmp	NextDigitBegin		
AllPrinted:
		ret
ShowDecimal	endp	

;
; MachZ-dependent Routine for calculating the processor speed. 
;
; The routine relies on the 8254 timer and accuracy of input clock(s) at 
; 14.132MHz and/or 48MHz inputs. The timer calibration loop is built up dependent
; of BS@SA5 setting (either for 14.132MHz or for 12MHz). The routine masks off all 
; IRQ's except timer and replaces timer IRQ with the predetermined handler.
;
; The measurement loop then executes the small subroutine of during 10 timer cycles
; (550ms). The number of clock ticks of 10 timer IRQ handler executions will be 
; added to the total clock counter afterwards.
;
; The CPU speed in MHz is then calculated with formula:
;	
;	CPU_MHz = ((count*1000)/550)/1000000
;		i.e.
;	CPU_MHz = count/550000
;
; Entry: None
;
; Exit: AX - speed in kHz*10 (hex)
;
; Uses: All
;

INT0_CONTROL	=	20h		; control port.
INT0_MASK	=	21h		; interrupt mask port.
INT1_CONTROL	=	0A0h		; control
INT1_MASK	=	0A1h		; interrupt mask port.
EOI		=	20h		; nonspecific EOI byte.

TIMER_BASE	=	40h		; 8253/8254 I/O base address.
TIMER_PORT_0	=	40h
TIMER_PORT_CMD	=	43h
TIMER_0_INIT14M	=	0FFFFh		; gives us 18.20...Hz
TIMER_0_INIT12M	= 	0D6D8h		; gives us exactly 18.18181818Hz

MeasureSpeed	proc

		; Replace timer IRQ
		
		cli				; all interrupts off
		push	0
		pop	ds
		mov	si, 8*4			; point to IRQ0 vector
		mov	ecx, ds:[si]
		mov	cs:[SystemTimer], ecx	; save original timer vector 
		mov	word ptr ds:[si], offset CertifiedTimer
		mov	word ptr ds:[si+2], cs	; set new, calibrated timer vector in place
		
		; Go mask off all other interrupts except timer
		
		; discard all pending IRQ's
		
		mov	al, EOI
		out	INT0_CONTROL, al
		out	INT1_CONTROL, al
		
		in	al, INT0_MASK
		mov	cs:[SystemIRQMaskLo], al
		mov	al, 11111010b			; enable cascade and timer only		
		call	Delay
		out	INT0_MASK, al
		call	Delay
		in	al, INT1_MASK
		mov	cs:[SystemIRQMaskHi], al
		mov	al, 11111111b			; disable everything on second PIC
		call	Delay
		out	INT1_MASK, al
		call	Delay
		
		; Set T0 now according to the 14MHz/12MHz clock source detected by BS@SA05.
		; We don't restore this later, because this must set up be the way
		; we do it here and if someone initializes it differently, it is
		; something they need to take special care again after this routine.
		
		mov	al, 62h
		mov	dx, 218h
		out	dx, al
		mov	dx, 21Ah
		in	eax, dx			; get bootstraps
		
		mov	bx, TIMER_0_INIT14M	; default for 14.318MHz clock
		test	eax, 100000b		; check for SA05
		jz	@f
		mov	bx, TIMER_0_INIT12M	; value for 12MHz timer input
@@:		
		mov	al, 00110110b
		out	TIMER_PORT_CMD, al	; initialize timer 0 for BIOS heartbeat.
        	call	Delay
		mov	ax, bx			; program limit to T0 to predefined value.
		out	TIMER_PORT_0, al	; program LSB.
	        call	Delay
		mov	al, ah			; (AL) = MSB to program T0 with.
		out	TIMER_PORT_0, al	; program MSB.
	        call	Delay
		
		; All set. Go with calculation loop now
		
		xor	ebx, ebx			; reset counter
		xor	ecx, ecx
		mov	dx, 11				; do 10 iterations + 1 for initial wait
		
		sti					; GOOOO!!!!
		
		;-------------
LOOP_CLOCKS = 7				; number of clocks we spend inside each iteration
@@:		
		add	ebx, ecx	; 1 clock
		cmp	dx, 0		; 1 clock
		jnz	@b		; 4 clocks
					; Somehow we loose one more clock!!!!!!
		;-------------
		cli					; STOPPP!!
		
		; EBX is now the counter value within 10 timer ticks.
		; We need to add the 10 times of Timer IRQ handler clock 
		; cycles as well.
		
		add	ebx, 10*35	; add number of clocks spent by timer interrupt vector
		push	ebx		; save count now
		
		; restore timer vector and IRQ masks
		
		mov	al, cs:[SystemIRQMaskHi]
		out	INT1_MASK, al
		jmp	$+2
		jmp	$+2		
		mov	al, EOI
		out	INT1_CONTROL, al
		mov	al, cs:[SystemIRQMaskLo]
		out	INT0_MASK, al		
		jmp	$+2
		jmp	$+2
		mov	al, EOI
		out	INT0_CONTROL, al
		
		mov	eax, cs:[SystemTimer]
		mov	ds:[si], eax
		
		; go with the normal system now
		
		sti
		
		pop	ebx
		mov	eax, ebx
		xor	edx, edx
		mov	ecx, 5500
		div	ecx		; edx:eax / ecx = speed
		
		ret
		
SystemTimer	dd	?
SystemIRQMaskLo	db	?
SystemIRQMaskHi	db	?
MeasureSpeed	endp

;
; This is a timer IRQ handler for speed calculation
;

CertifiedTimer	proc
		mov	ecx, LOOP_CLOCKS ; 1 clock	; set the substractor for speed loop
							; (number of clocks calibrated loop takes)
		dec	dx		 ; 1 clock	; substract the loop counter		
		mov	al, EOI		 ; 1 clock
		out	INT0_CONTROL, al ; 18 clocks
		iret			 ; 14 clocks
CertifiedTimer	endp

		include ..\BURAPI.ASM

HelloText	db	0Dh, 0Ah
		db	'STRAPS - MachZ Bootstraps Display Utility V1.00', 0Dh, 0Ah
		db	'-----------------------------------------------------------', 0
StrapText	db	'Bootstrap register value: ', 0
SpeedText	db	'Current CPU Speed: ', 0
MHzText		db	'MHz', 0
	
STRAP		STRUC
sBits		db	?
sMask		db	?
sName		db	40 dup(0)
sOption0	db	16 dup(0)
sOption1	db	16 dup(0)
sOption2	db	16 dup(0)
sOption3	db	16 dup(0)
STRAP		ENDS
				
Straps	label word
STRAP	<1, 1, '            Application Use 0 (SA00):'>
STRAPLEN = $-offset Straps
STRAP	<1, 1, '            Application Use 1 (SA01):'>
STRAP	<1, 1, '            Application Use 2 (SA02):'>
STRAP	<1, 1, '            Application Use 3 (SA03):'>
STRAP	<1, 1, '                     Reserved (SA04):'>
STRAP	<1, 1, '         14.3MHz clock source (SA05):', '(14MHz input)', '(48MHz/4)'>
STRAP	<1, 1, '           32kHz clock source (SA06):', '(32kHz input)', '(48MHz/1484)'>
STRAP	<1, 1, '            CPU core clocking (SA07):', '(Old-fashioned)', '(DLL)'>
STRAP	<1, 1, '                CPU raw clock (SA08):', '(Enabled)', '(Disabled)'>
STRAP	<1, 1, '               3rd PCI master (SA09):', '(Disabled)', '(Enabled)'>
STRAP	<1, 1, '                SIO test mode (SA10):', '(Disabled)', '(Enabled)'>
STRAP	<1, 1, '               BUR chipselect (SA11):', '(GPIO0)', '(Internal)'>
STRAP	<1, 1, '               Boot ROM width (SA12):', '(16-bit)', '(8-bit)'>
STRAP	<3, 7, '          CPU clock delay (SA15..13):'>
STRAP	<2, 3, '           CPU clock mode (SA17..16):', '(SYSCLK*1)', '(SYSCLK*2)', '(SYSCLK*4)', '(SYSCLK*3)'> 
STRAP	<1, 1, '          Frontside PCI clock (SA18):', '(SYSCLK/1)', '(SYSCLK/2)'>
STRAP	<1, 1, 'Backside PCI clock (BSPCICLK) (SA19):', '(SYSCLK/1)', '(SYSCLK/2)'>
STRAP	<1, 1, '    Backside PCI clock source (SA20):', '(PCI Bus)', '(BSPCICLK)'>
STRAP	<1, 1, '                USB test mode (SA21):', '(Disabled)', '(Enabled)'>
STRAP	<1, 1, '                   ZFL enable (SA22):', '(Disabled)', '(Enabled)'>
STRAP	<1, 1, '                   BUR enable (SA23):', '(Disabled)', '(Enabled)'>

		db	0FFh		; end-of-straps marker
		
END START
ENDS

